# Generated by Django 3.2.20 on 2023-11-06 20:00

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Mock',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('index', models.IntegerField()),
                ('headers', models.JSONField()),
                ('content', models.JSONField()),
                ('status', models.IntegerField()),
                ('method', models.CharField(max_length=6)),
                ('pattern', models.CharField(max_length=1000)),
                ('spec_name', models.CharField(db_index=True, max_length=100)),
            ],
        ),
    ]
