# Generated by Django 3.2.25 on 2024-06-30 16:41

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('dbpreset', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='mock',
            name='created_at',
            field=models.DateTimeField(auto_now_add=True, default=django.utils.timezone.now),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='mock',
            name='env',
            field=models.CharField(db_index=True, default='', max_length=100),
        ),
    ]
